/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.my.client;

import com.my.SSLContextFactory;
import com.my.server.ControllerUtils;
import com.sun.grizzly.Context;
import com.sun.grizzly.Controller;
import com.sun.grizzly.IOEvent;
import com.sun.grizzly.SSLCallbackHandler;
import com.sun.grizzly.SSLConnectorHandler;
import com.sun.grizzly.SSLSelectorHandler;
import com.sun.grizzly.util.SSLUtils;
import com.sun.grizzly.util.SelectorFactory;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;
import javax.net.ssl.SSLContext;

/**
 *
 * @author oleksiys
 */
public class ClientTester {

    private AtomicInteger counter = new AtomicInteger();
    private ArrayBlockingQueue<Pair<SSLConnectorHandler, ByteBuffer>> connections;
    private SSLContext sslContext;
    private Controller controller;

    public static void main(String[] args) throws Exception {
        ClientTester tester = new ClientTester();
        tester.init();
        tester.prepareClients(5);
        tester.runTests(10, 1000000);
        tester.stopClients();
        tester.stop();
    }

    public void runTests(int numberOfThreads, int numberOfExecutions) {
        try {
            SelectorFactory.setMaxSelectors(numberOfThreads * 3);
        } catch (Exception e) {
            e.printStackTrace();
        }

        int runnersCount = numberOfThreads * 2;
        final ArrayBlockingQueue<Runnable> runners = new ArrayBlockingQueue<Runnable>(runnersCount);
        for (int i = 0; i < runnersCount; i++) {
            Runnable runner = new Runnable() {

                public void run() {
                    try {
                        Pair<SSLConnectorHandler, ByteBuffer> pair = connections.take();
                        try {
                            testClient(pair);
                        } catch (Exception e) {
                            e.printStackTrace();
                        } finally {
                            pair.elem2.clear();
                            connections.offer(pair);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                    } finally {
                        runners.add(this);
                    }
                }
            };

            runners.add(runner);
        }

        ExecutorService executor = Executors.newFixedThreadPool(numberOfThreads);
        for (int i = 0; i < numberOfExecutions; i++) {
            Runnable runner = null;
            try {
                runner = runners.take();
                executor.execute(runner);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }

        executor.shutdown();
        try {
            executor.awaitTermination(30, TimeUnit.SECONDS);
        } catch (Exception e) {
            e.printStackTrace();
        }

        System.out.println("SHUTING DOWN...........");
    }

    public void init() throws Exception {
        sslContext = SSLContextFactory.createSSLContext();
        controller = createSSLController();
        ControllerUtils.startController(controller);
    }

    public void stop() {
        ControllerUtils.stopController(controller);
    }

    public void prepareClients(int number) throws IOException {
        connections = new ArrayBlockingQueue<Pair<SSLConnectorHandler, ByteBuffer>>(number);
        for (int i = 0; i < number; i++) {
            SSLConnectorHandler handler = createClient(controller, sslContext);
            ByteBuffer bb = ByteBuffer.allocate(32000);
            handler.handshake(bb, true);

            connections.offer(new Pair<SSLConnectorHandler, ByteBuffer>(handler, bb));
        }
    }

    public void stopClients() {
        for (Pair<SSLConnectorHandler, ByteBuffer> pair : connections) {
            try {
                pair.elem1.close();
            } catch (Exception e) {
            }
        }
    }

    public void testClient(Pair<SSLConnectorHandler, ByteBuffer> pair) throws Exception {
        SSLConnectorHandler handler = pair.elem1;
        ByteBuffer bb = pair.elem2;

        String testString = "Hello#" + counter.incrementAndGet();
        bb.put(testString.getBytes());
        bb.flip();
        handler.write(bb, true);
        bb.clear();

        long bytesRead = 1;

        while (bytesRead > 0 && bb.position() < testString.length()) {
            bytesRead = handler.read(bb, true);
        }

        if (bytesRead <= 0) {
            System.out.println("NO REPLY COME. Channel: " + handler.getUnderlyingChannel() + " bytesRead: " + bytesRead + " bb: " + bb + " ReadTimeout: " + SSLUtils.getReadTimeout());
        } else {
            bb.flip();
            String replyString = new String(bb.array(), bb.arrayOffset(), bb.remaining());
            if (!replyString.equals(testString)) {
                System.out.println("RETURNED data doesn't match. Channel: " + handler.getUnderlyingChannel() + " Reply: " + replyString + " request: " + testString);
            }
        }

        bb.clear();
    }

    private SSLConnectorHandler createClient(Controller controller, SSLContext context) throws IOException {
        final CountDownLatch latch = new CountDownLatch(1);
        final SSLConnectorHandler sslConnector =
                (SSLConnectorHandler) controller.acquireConnectorHandler(
                Controller.Protocol.TLS);

        SSLCallbackHandler callbackHandler = new SSLCallbackHandler<Context>() {

            public void onHandshake(IOEvent<Context> event) {
                System.out.println("onHandshake");
            }

            public void onConnect(IOEvent<Context> event) {
                try {
                    sslConnector.finishConnect(event.attachment().getSelectionKey());
                } catch (Exception e) {
                    e.printStackTrace();
                }
                latch.countDown();
            }

            public void onRead(IOEvent<Context> event) {
                System.out.println("onRead");
            }

            public void onWrite(IOEvent<Context> event) {
                System.out.println("onWrite");
            }
        };

        sslConnector.setSSLContext(context);
        sslConnector.connect(new InetSocketAddress("localhost", 5000), callbackHandler);

        try {
            latch.await();
        } catch (InterruptedException ex) {
        }
        return sslConnector;
    }

    private Controller createSSLController() {
        SSLSelectorHandler selectorHandler = new SSLSelectorHandler(true);

        final Controller controller = new Controller();

        controller.setSelectorHandler(selectorHandler);
        controller.setHandleReadWriteConcurrently(false);

        return controller;
    }

    public class Pair<A, B> {

        A elem1;
        B elem2;

        public Pair(A elem1, B elem2) {
            this.elem1 = elem1;
            this.elem2 = elem2;
        }
    }
}
