package com.my.server;

import com.my.SSLContextFactory;
import com.my.server.grizzly.MySSLProcessor;
import com.my.server.grizzly.MySSLSelectorHandler;
import com.sun.grizzly.*;
import com.sun.grizzly.filter.SSLReadFilter;
import com.sun.grizzly.util.OutputWriter;

import javax.net.ssl.SSLContext;
import java.io.IOException;
import java.util.Hashtable;

public class Server {

    public static org.apache.log4j.Logger logger = org.apache.log4j.Logger.getLogger(Server.class);
    public static boolean USE_GRIZZLY = true;
    public static boolean USE_GRIZZLY_ASYNC_WRITES = true;
    private static final int GRIZZLY_READ_THREADS_COUNT = System.getProperty("GRIZZLY_READ_THREADS_COUNT") == null ? 0 : Integer.parseInt(System.getProperty("GRIZZLY_READ_THREADS_COUNT"));
    private Hashtable<Integer, Controller> serverControllers = new Hashtable<Integer, Controller>();

    public static void main(String[] args) throws Exception {
        Server server = new Server();
        Controller controller = server.createSSLSocket(5000);
        try {
            ControllerUtils.startController(controller);

            System.out.println("Press any key to exit...");
            System.in.read();
        } finally {
            ControllerUtils.stopController(controller);
        }

    }

    public Controller createSSLSocket(int port) throws Exception {
        if (logger.isInfoEnabled()) {
            logger.info("Server.createSSLSocket:USING GRIZZLY !!!");
        }
        if (logger.isDebugEnabled()) {
            logger.debug("Server.createSSLSocket:Initialized SSLSelectorHandler with port " + port + "!!!");
        }

        final SSLContext context = SSLContextFactory.createSSLContext();

        Controller controller = new Controller();
        controller.setProtocolChainInstanceHandler(new DefaultProtocolChainInstanceHandler() {

            public ProtocolChain poll() {
                ProtocolChain protocolChain = protocolChains.poll();
                if (protocolChain == null) {
                    protocolChain = new DefaultProtocolChain();
                    SSLReadFilter readFilter = new SSLReadFilter();
                    readFilter.setEnabledProtocols(SSLContextFactory.RECOMMENDED_PROTOCOLS);
                    readFilter.setSSLContext(context);
                    
                    MySSLProcessor mysslProcessor = new MySSLProcessor(Server.this, false);
//                        mysslReadFilter.setEnabledCipherSuites(RECOMMENDED_CIPHER_SUITES);
                    protocolChain.addFilter(readFilter);
                    protocolChain.addFilter(mysslProcessor);
                }
                return protocolChain;
            }
        });
        controller.setHandleReadWriteConcurrently(true);
        SSLSelectorHandler selectorHandler = new MySSLSelectorHandler();
        selectorHandler.setPort(port);
        if (logger.isDebugEnabled()) {
            logger.debug("Server.createSSLSocket:Initialized SSLSelectorHandler!!!");
        }
        controller.setSelectorHandler(selectorHandler);
        // 0 count for now
        controller.setReadThreadsCount(GRIZZLY_READ_THREADS_COUNT);
        OutputWriter.setDefaultWriteTimeout(5000);
        serverControllers.put(port, controller);
        if (logger.isDebugEnabled()) {
            logger.debug("Server.createSSLSocket:Starting Controller ...");
        }
        return controller;
    }

    public boolean stopListener(int port) {
        if (USE_GRIZZLY) {
            Controller controller = serverControllers.get(port);
            if (controller != null) {
                serverControllers.remove(port);
                try {
                    controller.stop();
                } catch (IOException e) {
                    return false;
                }
                return true;
            } else {
                return false;
            }
        } else {
            // Unbind the regular C Lib based SSL Server Socket
        }
        return true;
    }

    public void addConHandler(ServerConnectionHandler conHandler) {
        // Add connection handler
    }
}