package com.my.server;

import com.my.server.grizzly.MySSLAsyncQueueWriter;
import com.sun.grizzly.TCPSelectorHandler;
import com.sun.grizzly.util.SSLOutputWriter;
import com.sun.grizzly.util.WorkerThread;
import org.apache.log4j.Logger;

import javax.net.ssl.SSLEngine;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.ClosedChannelException;
import java.nio.channels.SelectionKey;
import java.nio.channels.SocketChannel;

public class ServerConnectionHandler {

    private SocketChannel channel;
    private SSLEngine sslEngine;
    private SelectionKey selectionKey;
    private TCPSelectorHandler selectorHandler;
    private static final Logger logger = Logger.getLogger(ServerConnectionHandler.class);
    private static final int GRIZZLY_DEFAULT_OUTPUT_BB_SIZE = 5 * 4096;

    public ServerConnectionHandler(TCPSelectorHandler selectorHandler, SelectionKey selectionKey, SocketChannel channel, SSLEngine engine) {
        this.selectorHandler = selectorHandler;
        this.selectionKey = selectionKey;
        this.channel = channel;
        this.sslEngine = engine;
    }

    public void read(ByteBuffer byteBuffer) {
        // Parse the bytebuffer data and pass the constructed objects to the business processing layer.
    }

    public void sendBytes(byte[] b1, String msgID) {
        if (Server.USE_GRIZZLY) {
            if (channel == null || b1 == null) {
                return;
            }

            if (logger.isDebugEnabled()) {
                logger.debug("ServerConnectionHandler.sendBytes:In here ...");
            }

            if (Server.USE_GRIZZLY_ASYNC_WRITES) {
                try {
//                    System.out.println("SendBytes size: " + b1.length + " sslEngine: " + sslEngine);
                    MySSLAsyncQueueWriter writer = (MySSLAsyncQueueWriter) selectorHandler.getAsyncQueueWriter();
                    writer.write(selectionKey, ByteBuffer.wrap(b1), sslEngine);
                } catch (ClosedChannelException e) {
                    e.printStackTrace();
                    logger.error(e.getMessage());
                } catch (IOException e) {
                    e.printStackTrace();
                    logger.error(e.getMessage());
                } catch (Exception e) {
                    e.printStackTrace();
                }
            } else {
                try {
                    if (channel == null) {
                        return;
                    }
                    WorkerThread worker = (WorkerThread) Thread.currentThread();
                    ByteBuffer bb = worker.getByteBuffer();
                    ByteBuffer outputBB = worker.getOutputBB();
                    bb.clear();
                    bb.put(b1);
                    bb.flip();
                    long n = SSLOutputWriter.flushChannel(channel, bb, outputBB, sslEngine);
                    System.out.println("Written: " + n + " bb: " + bb);
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        } else {
            // Send bytes using regular sockets
        }
    }
}