package com.my.server.grizzly;

import com.sun.grizzly.SelectorHandler;
import com.sun.grizzly.async.TCPAsyncQueueWriter;
import com.sun.grizzly.util.SSLOutputWriter;
import org.apache.log4j.Logger;

import javax.net.ssl.SSLEngine;
import java.io.IOException;
import java.net.SocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * Date: Jul 14, 2008
 * Time: 4:19:13 PM
 */
public class MySSLAsyncQueueWriter extends TCPAsyncQueueWriter {
    private static Logger logger = Logger.getLogger(MySSLAsyncQueueWriter.class);
    private static final int GRIZZLY_DEFAULT_OUTPUT_BB_SIZE = 5 * 4096;

    private static final ConcurrentHashMap<Channel, ChannelRegistrationRecord> channelRegistrationMap = new ConcurrentHashMap<Channel, ChannelRegistrationRecord>();

    private class ChannelRegistrationRecord {
        private AtomicInteger writeCount = new AtomicInteger(1);
        private SSLEngine sslEngine;

        private ChannelRegistrationRecord(SSLEngine sslEngine) {
            this.sslEngine = sslEngine;
        }
    }

    public MySSLAsyncQueueWriter(SelectorHandler selectorHandler) {
        super(selectorHandler);
        logger.error("Creating MySSLAsyncQueueWriter");
    }

    public void write(SelectionKey key, ByteBuffer buffer, SSLEngine sslEngine) throws IOException {
        ChannelRegistrationRecord record = channelRegistrationMap.putIfAbsent(key.channel(), new ChannelRegistrationRecord(sslEngine));
        if (record != null) {
            record.writeCount.incrementAndGet();
        }
        super.write(key, buffer);
    }

    protected void doWrite(WritableByteChannel channel, SocketAddress dstAddress, ByteBuffer byteBuffer) throws IOException {
        SSLEngine sslEngine;
        ChannelRegistrationRecord record = channelRegistrationMap.get(channel);
        if (record == null) {
            logger.error("Channel is not registered for an async write");
            return;
        }
        sslEngine = record.sslEngine;
        if (sslEngine == null) {
            logger.error("SSLEngine is null.");
            return;
        }
        if (record.writeCount.intValue() > 200) {
            ((SocketChannel)channel).socket().close();
            channel.close();
            channelRegistrationMap.remove(channel);
            return;
        }
        if (record.writeCount.decrementAndGet() == 0) {
            channelRegistrationMap.remove(channel);
        }
        SSLOutputWriter.flushChannel((SelectableChannel) channel, byteBuffer, ByteBuffer.allocate(Math.max(GRIZZLY_DEFAULT_OUTPUT_BB_SIZE, sslEngine.getSession().getPacketBufferSize())), sslEngine);
    }
}