package com.my.server.grizzly;

import com.sun.grizzly.Context;
import com.sun.grizzly.SSLSelectorHandler;
import org.apache.log4j.Logger;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.nio.channels.SelectableChannel;
import java.nio.channels.SelectionKey;
import java.nio.channels.SocketChannel;

/**
 * Date: Jun 12, 2008
 * Time: 3:45:22 PM
 */
public class MySSLSelectorHandler extends SSLSelectorHandler {
    protected static Logger logger = Logger.getLogger(MySSLSelectorHandler.class);
    private int socketCount = 0;

    public MySSLSelectorHandler() {
        super();
    }

    public void preSelect(Context ctx) throws IOException {
        if (asyncQueueWriter == null) {
            asyncQueueWriter = new MySSLAsyncQueueWriter(this);
        }
        super.preSelect(ctx);
    }

    public boolean onAcceptInterest(SelectionKey key, Context ctx) throws IOException {
        SocketChannel socketChannel = (SocketChannel) acceptWithoutRegistration(key);
        if (socketChannel != null) {
            Socket socket = socketChannel.socket();
            InetSocketAddress socketAddress = (InetSocketAddress) socket.getRemoteSocketAddress();
            String clientIP = socketAddress.getAddress().getHostAddress();
            boolean allowConnection = FloodGuard.getInstance().allowConnectionFromIP(clientIP);
            if (allowConnection) {
                logger.info("onAcceptInterest():Accepting connection from - " + clientIP);
                this.socketCount++;
                configureChannel(socketChannel);
                SelectionKey readKey =
                        socketChannel.register(selector, SelectionKey.OP_READ);
                readKey.attach(System.currentTimeMillis());
                return true;
            } else {
                logger.error("onAcceptInterest():Rejecting connection from - " + clientIP);
                socket.close();
                socketChannel.close();
            }
        }
        return false;
    }

    public void closeChannel(SelectableChannel channel) {
        if (channel != null) {
            SocketChannel socketChannel = (SocketChannel) channel;
            InetSocketAddress socketAddress = (InetSocketAddress) socketChannel.socket().getRemoteSocketAddress();
            if (socketAddress != null) {
                String clientIP = socketAddress.getAddress().getHostAddress();
                if (clientIP != null) {
                    FloodGuard.getInstance().decrementCounts(clientIP);
                }
            }
        }
        socketCount--;
        super.closeChannel(channel);
    }

    public int getSocketCount() {
        return socketCount;
    }
}